/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package FunctionLayer;

import DataLayer.DataMapper;
import FunctionLayer.StanfordParser.SentimentAnalyzerTest;
import com.google.common.base.Stopwatch;
import com.google.common.collect.MapMaker;
import edu.stanford.nlp.ie.AbstractSequenceClassifier;
import edu.stanford.nlp.ie.crf.CRFClassifier;
import edu.stanford.nlp.ling.CoreLabel;
import edu.stanford.nlp.ling.HasWord;
import edu.stanford.nlp.ling.TaggedWord;
import edu.stanford.nlp.ling.Word;
import edu.stanford.nlp.parser.lexparser.LexicalizedParser;
import edu.stanford.nlp.parser.shiftreduce.ShiftReduceParser;
import edu.stanford.nlp.pipeline.Annotation;
import edu.stanford.nlp.pipeline.StanfordCoreNLP;
import edu.stanford.nlp.process.DocumentPreprocessor;
import edu.stanford.nlp.tagger.maxent.MaxentTagger;
import edu.stanford.nlp.trees.GrammaticalStructureFactory;
import edu.stanford.nlp.trees.Tree;
import edu.stanford.nlp.trees.TreebankLanguagePack;
import java.io.IOException;
import java.io.StringReader;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Properties;
import java.util.Random;
import java.util.concurrent.Callable;
import java.util.concurrent.ConcurrentMap;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.ForkJoinPool;
import java.util.concurrent.Future;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.TimeoutException;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 *
 * @author install1
 */
public class Datahandler {

    public static final long EXPIRE_TIME_IN_SECONDS = TimeUnit.SECONDS.convert(6, TimeUnit.MINUTES);
    public static final long EXPIRE_TIME_IN_SECONDS1 = TimeUnit.SECONDS.convert(10, TimeUnit.HOURS);
    public static Datahandler instance = new Datahandler();
    private volatile boolean refreshMatrixFromDB;
    private static int secondaryIterator = 0;
    private final ConcurrentMap<Integer, String> stringCache;
    private static ConcurrentMap<String, Annotation> pipelineAnnotationCache;
    private static ConcurrentMap<String, Annotation> pipelineSentimentAnnotationCache;
    private static ConcurrentMap<String, Annotation> jmweAnnotationCache;
    private LinkedHashMap<String, LinkedHashMap<String, Double>> lHMSMX = new LinkedHashMap();
    private final Stopwatch stopwatch;
    private final Stopwatch stopwatch1;
    private ForkJoinPool executor;
    private static String shiftReduceParserPath = "edu/stanford/nlp/models/srparser/englishSR.ser.gz";
    private static String sentimentModel = "edu/stanford/nlp/models/sentiment/sentiment.ser.gz";
    private static String lexParserEnglishRNN = "edu/stanford/nlp/models/lexparser/englishRNN.ser.gz";
    private static String taggerPath = "edu/stanford/nlp/models/pos-tagger/english-left3words/english-left3words-distsim.tagger";
    private static String nerModel = "edu/stanford/nlp/models/ner/english.all.3class.distsim.crf.ser.gz";
    private static MaxentTagger tagger;
    private static ShiftReduceParser model;
    private static String[] options = {"-maxLength", "100"};
    private static Properties props = new Properties();
    private static Properties propsSentiment = new Properties();
    private static GrammaticalStructureFactory gsf;
    private static LexicalizedParser lp;
    private static TreebankLanguagePack tlp;
    private static AbstractSequenceClassifier<CoreLabel> classifier;
    // set up Stanford CoreNLP pipeline
    private static StanfordCoreNLP pipeline;
    private static StanfordCoreNLP pipelineSentiment;

    public Datahandler() {
        this.stopwatch = Stopwatch.createUnstarted();
        this.stopwatch1 = Stopwatch.createStarted();
        this.stringCache = new MapMaker().concurrencyLevel(2).makeMap();
        //cant sadly just have one pipelines for every annotation, one pipeline per annotation is required
        this.jmweAnnotationCache = new MapMaker().concurrencyLevel(2).makeMap();
        this.pipelineAnnotationCache = new MapMaker().concurrencyLevel(2).makeMap();
        this.pipelineSentimentAnnotationCache = new MapMaker().concurrencyLevel(2).makeMap();
    }

    public void shiftReduceParserInitiate() {
        //got 8 cores
        CountDownLatch cdl = new CountDownLatch(4);
        new Thread(() -> {
            model = ShiftReduceParser.loadModel(shiftReduceParserPath, options);
            cdl.countDown();
        }).start();
        new Thread(() -> {
            try {
                classifier = CRFClassifier.getClassifierNoExceptions(nerModel);
            } catch (ClassCastException ex) {
                Logger.getLogger(Datahandler.class.getName()).log(Level.SEVERE, null, ex);
            }
            cdl.countDown();
        }).start();
        new Thread(() -> {
            propsSentiment.setProperty("parse.model", lexParserEnglishRNN);
            propsSentiment.setProperty("ner.model", nerModel);
            propsSentiment.setProperty("sentiment.model", sentimentModel);
            propsSentiment.setProperty("parse.maxlen", "100");
            propsSentiment.setProperty("annotators", "tokenize,ssplit,pos,parse,depparse,sentiment"); //coref too expensive memorywise
            propsSentiment.setProperty("tokenize.options", "untokenizable=firstDelete");
            pipelineSentiment = new StanfordCoreNLP(propsSentiment);
            tagger = new MaxentTagger(taggerPath);
            cdl.countDown();
        }).start();
        new Thread(() -> {
            props.setProperty("parse.model", shiftReduceParserPath);
            props.setProperty("parse.maxlen", "100");
            props.setProperty("parse.binaryTrees", "true");
            props.setProperty("annotators", "tokenize,ssplit,pos,lemma,parse");
            props.setProperty("tokenize.options", "untokenizable=firstDelete");
            pipeline = new StanfordCoreNLP(props);
            cdl.countDown();
        }).start();
        lp = LexicalizedParser.loadModel(lexParserEnglishRNN, options);
        tlp = lp.getOp().langpack();
        gsf = tlp.grammaticalStructureFactory();
        try {
            cdl.await();
        } catch (InterruptedException ex) {
            System.out.println("cdl await interrupted: " + ex.getLocalizedMessage() + "\n");
        }
        System.out.println("finished shiftReduceParserInitiate\n");
    }

    public static AbstractSequenceClassifier<CoreLabel> getClassifier() {
        return classifier;
    }

    public static void setClassifier(AbstractSequenceClassifier<CoreLabel> classifier) {
        Datahandler.classifier = classifier;
    }

    public void updateStringCache() {
        try {
            checkIfUpdateStrings(true);
        } catch (CustomError ex) {
            Logger.getLogger(Datahandler.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    public void instantiateExecutor() {
        this.executor = new ForkJoinPool(25,
                ForkJoinPool.defaultForkJoinWorkerThreadFactory,
                null, false);
    }

    public static GrammaticalStructureFactory getGsf() {
        return gsf;
    }

    public static StanfordCoreNLP getPipeline() {
        return pipeline;
    }

    public static StanfordCoreNLP getPipelineSentiment() {
        return pipelineSentiment;
    }

    public static MaxentTagger getTagger() {
        return tagger;
    }

    public static ShiftReduceParser getModel() {
        return model;
    }

    private Map<Integer, String> getCache() throws SQLException, IOException, CustomError {
        return DataMapper.getAllStrings();
    }

    public int getlHMSMXSize() {
        return lHMSMX.size();
    }

    public int getstringCacheSize() {
        return stringCache.size();
    }

    public void initiateMYSQL() throws SQLException, IOException {
        try {
            DataMapper.createTables();
            stringCache.putAll(getCache());
            lHMSMX = DataMapper.getAllRelationScores();
        } catch (CustomError ex) {
            Logger.getLogger(Datahandler.class
                    .getName()).log(Level.SEVERE, null, ex);
        }
    }

    public void addHLstatsMessages() {
        ConcurrentMap<Integer, String> hlStatsMessages = DataMapper.getHLstatsMessages();
        ConcurrentMap<Integer, String> strCacheLocal = stringCache;
        //might want a hardcap
        hlStatsMessages.values().parallelStream().forEach(str -> {
            if (!str.startsWith("!")) {
                String orElse = strCacheLocal.values().parallelStream().filter(e -> e.equals(str)).findAny().orElse(null);
                if (orElse == null) {
                    MessageResponseHandler.getMessage(str);
                }
            }
        });
    }

    public void instantiateAnnotationMapJMWE() {
        if (!stringCache.isEmpty()) {
            ConcurrentMap<String, Annotation> jmweAnnotation = PipelineJMWESingleton.INSTANCE.getJMWEAnnotation(stringCache.values());
            for (Entry<String, Annotation> entries : jmweAnnotation.entrySet()) {
                jmweAnnotationCache.put(entries.getKey(), entries.getValue());
            }
        }
    }

    public void instantiateAnnotationMap() {
        if (!stringCache.isEmpty()) {
            stringCache.values().parallelStream().forEach(str -> {
                System.out.println("str annotation pipeline pipelinesentiment: " + str + "\n");
                Annotation strAnno = new Annotation(str);
                pipeline.annotate(strAnno);
                pipelineAnnotationCache.put(str, strAnno);
                Annotation strAnno2 = new Annotation(str);
                pipelineSentiment.annotate(strAnno2);
                pipelineSentimentAnnotationCache.put(str, strAnno2);
            });
        }
    }

    public synchronized void checkIfUpdateMatrixes() {
        refreshMatrixFromDB = false;
        if (stopwatch1.elapsed(TimeUnit.SECONDS) >= EXPIRE_TIME_IN_SECONDS1) {
            refreshMatrixFromDB = true;
            lHMSMX = DataMapper.getAllRelationScores();
            stopwatch1.reset();
        }
        //requiring atleast 10 entries ensures no issues in case of empty stringcache
        if (stringCache.values().size() > 10 && !refreshMatrixFromDB) {
            ConcurrentMap<Integer, String> stringCachelocal = stringCache;
            int selectUpdate = -1;
            LinkedHashMap<String, LinkedHashMap<String, Double>> LHMSMXLocal = lHMSMX;
            int ij2 = 0;
            for (String str : stringCachelocal.values()) {
                boolean updatepresent = false;
                for (String strlocal : LHMSMXLocal.keySet()) {
                    if (strlocal.equals(str)) {
                        updatepresent = true;
                        break;
                    }
                }
                if (!updatepresent) {
                    selectUpdate = ij2;
                    break;
                }
                ij2++;
            }
            if (selectUpdate == -1 || selectUpdate + 1 == stringCachelocal.size()) {
                int valueSize = stringCachelocal.size();
                if (secondaryIterator + 1 >= valueSize) {
                    secondaryIterator = 0;
                }
                selectUpdate = secondaryIterator;
                secondaryIterator++;
            }
            ConcurrentMap<Integer, String> strIndexNavigator = new MapMaker().concurrencyLevel(2).makeMap();
            String get = stringCachelocal.getOrDefault(selectUpdate, null);
            if (get == null) {
                get = stringCachelocal.get(new Random().nextInt(stringCachelocal.size() - 1));
            }
            strIndexNavigator.put(0, get);
            ConcurrentMap<Integer, SimilarityMatrix> matrixUpdateList = new MapMaker().concurrencyLevel(2).makeMap();
            ConcurrentMap<Integer, Future<SimilarityMatrix>> futures = new MapMaker().concurrencyLevel(2).makeMap();
            strIndexNavigator.values().forEach((str) -> {
                stringCachelocal.values().stream().filter((str1) -> (!str.equals(str1))).forEachOrdered((str1) -> {
                    boolean present = false;
                    LinkedHashMap<String, Double> orDefault = lHMSMX.getOrDefault(str, null);
                    if (orDefault != null) {
                        for (String strkey : orDefault.keySet()) {
                            if (strkey.equals(str1)) {
                                present = true;
                                break;
                            }
                        }
                    }
                    if (!present) {
                        orDefault = lHMSMX.getOrDefault(str1, null);
                        if (orDefault != null) {
                            for (String strkey : orDefault.keySet()) {
                                if (strkey.equals(str)) {
                                    present = true;
                                    break;
                                }
                            }
                        }
                    }
                    if (!present) {
                        LinkedHashMap<String, Double> orDefault1 = lHMSMX.getOrDefault(str, null);
                        if (orDefault1 == null) {
                            orDefault1 = new LinkedHashMap<String, Double>();
                        }
                        orDefault1.put(str1, 0.0);
                        lHMSMX.put(str, orDefault1);
                        SimilarityMatrix SMX = new SimilarityMatrix(str, str1);
                        Callable<SimilarityMatrix> worker = new SentimentAnalyzerTest(str, str1, SMX, jmweAnnotationCache.get(str),
                                jmweAnnotationCache.get(str1), pipelineAnnotationCache.get(str), pipelineAnnotationCache.get(str1),
                                pipelineSentimentAnnotationCache.get(str), pipelineSentimentAnnotationCache.get(str1));
                        futures.put(futures.size() + 1, executor.submit(worker));
                    }
                });
            });
            System.out.println("finished worker assignment, futures size: " + futures.size() + "\n");
            for (Future<SimilarityMatrix> future : futures.values()) {
                SimilarityMatrix SMX = new SimilarityMatrix("", "");
                try {
                    SMX = future.get(20, TimeUnit.SECONDS);
                } catch (InterruptedException | ExecutionException | TimeoutException ex) {
                    Logger.getLogger(Datahandler.class.getName()).log(Level.SEVERE, null, ex);
                    SMX = null;
                }
                if (SMX != null) {
                    LinkedHashMap<String, Double> getFuture = lHMSMX.getOrDefault(SMX.getPrimaryString(), null);
                    getFuture.put(SMX.getSecondaryString(), SMX.getDistance());
                    lHMSMX.put(SMX.getPrimaryString(), getFuture);
                    matrixUpdateList.put(matrixUpdateList.size() + 1, SMX);
                }
            }
            try {
                if (!matrixUpdateList.isEmpty()) {
                    DataMapper.insertSementicMatrixes(matrixUpdateList);
                    System.out.println("finished datamapper semetic insert");
                }
            } catch (CustomError ex) {
                Logger.getLogger(Datahandler.class
                        .getName()).log(Level.SEVERE, null, ex);
            }
        }
    }

    public synchronized void checkIfUpdateStrings(boolean hlStatsMsg) throws CustomError {
        if (stopwatch.elapsed(TimeUnit.SECONDS) >= EXPIRE_TIME_IN_SECONDS || !stopwatch.isRunning()) {
            ConcurrentMap<Integer, String> str = MessageResponseHandler.getStr();
            str = cutContent(str, hlStatsMsg);
            str = filterContent(str);
            str = removeSlacks(str);
            System.out.println("finished removeSlacks \n" + str.size() + "\n");
            str = verifyCalculationFitness(str);
            System.out.println("Check if updateString str size POST: " + str.size() + "\n");
            try {
                DataMapper.InsertMYSQLStrings(str);
            } catch (CustomError ex) {
                Logger.getLogger(Datahandler.class
                        .getName()).log(Level.SEVERE, null, ex);
            }
            MessageResponseHandler.setStr(new MapMaker().concurrencyLevel(2).makeMap());
            if (!stopwatch.isRunning()) {
                stopwatch.start();
            } else {
                stopwatch.reset();
            }
        }
    }

    public synchronized String getResponseMsg(String str) throws CustomError {
        str = str.trim();
        if (str.startsWith("<@")) {
            str = str.substring(str.indexOf("> ") + 2);
        }
        final LinkedHashMap<String, LinkedHashMap<String, Double>> LHMSMXLocal = lHMSMX;
        ConcurrentMap<Integer, String> strArrs = stringCache;
        double Score = -10000;
        SimilarityMatrix SMXreturn = new SimilarityMatrix("", "");
        System.out.println("pre mostSimilarSTR \n");
        String mostSimilarSTR = mostSimilar(str, strArrs);
        if (mostSimilarSTR != null) {
            System.out.println("mostSimilarSTR; " + mostSimilarSTR + "\n");
            LinkedHashMap<String, Double> orDefault = LHMSMXLocal.getOrDefault(mostSimilarSTR, null);
            if (orDefault != null) {
                for (Entry<String, Double> entrySet : orDefault.entrySet()) {
                    double smxDistance = entrySet.getValue();
                    if (smxDistance > Score) {
                        Score = smxDistance;
                        SMXreturn = new SimilarityMatrix(mostSimilarSTR, entrySet.getKey(), smxDistance);
                    }
                }
            }
            for (Entry<String, LinkedHashMap<String, Double>> values1 : LHMSMXLocal.entrySet()) {
                LinkedHashMap<String, Double> value = values1.getValue();
                for (Entry<String, Double> keystr : value.entrySet()) {
                    if (keystr.getKey().equals(mostSimilarSTR)) {
                        double smxDistance = keystr.getValue();
                        if (smxDistance > Score) {
                            Score = smxDistance;
                            SMXreturn = new SimilarityMatrix(values1.getKey(), keystr.getKey(), smxDistance);
                        }
                    }
                }
            }
            if (!SMXreturn.getPrimaryString().isEmpty()) {
                if (SMXreturn.getPrimaryString().equals(mostSimilarSTR)) {
                    return SMXreturn.getSecondaryString();
                } else {
                    return SMXreturn.getPrimaryString();
                }
            }
        }
        System.out.println("none within 8 range");
        Annotation strAnno = new Annotation(str);
        pipeline.annotate(strAnno);
        Annotation strAnno2 = new Annotation(str);
        pipelineSentiment.annotate(strAnno2);
        List<String> notactualList = new ArrayList();
        notactualList.add(str);
        ConcurrentMap<String, Annotation> jmweAnnotation = PipelineJMWESingleton.INSTANCE.getJMWEAnnotation(notactualList);
        final Annotation strAnno3 = jmweAnnotation.values().iterator().next();
        ConcurrentMap<Integer, String> strCache = stringCache;
        ConcurrentMap<Integer, Future<SimilarityMatrix>> futureslocal = new MapMaker().concurrencyLevel(2).makeMap();
        ConcurrentMap<Integer, SimilarityMatrix> futurereturn = new MapMaker().concurrencyLevel(2).makeMap();
        String strF = str;
        strCache.values().parallelStream().forEach((str1) -> {
            if (!strF.equals(str1)) {
                SimilarityMatrix SMX = new SimilarityMatrix(strF, str1);
                Callable<SimilarityMatrix> worker = new SentimentAnalyzerTest(strF, str1, SMX,
                        strAnno3, jmweAnnotationCache.get(str1), strAnno,
                        pipelineAnnotationCache.get(str1), strAnno2,
                        pipelineSentimentAnnotationCache.get(str1));
                futureslocal.put(futureslocal.size() + 1, executor.submit(worker));
            }
        });
        int index = 0;
        futureslocal.values().parallelStream().forEach((future) -> {
            SimilarityMatrix SMX = new SimilarityMatrix("", "");
            try {
                SMX = future.get(20, TimeUnit.SECONDS);
                futurereturn.put(futurereturn.size() + 1, SMX);
            } catch (InterruptedException | ExecutionException | TimeoutException ex) {
                System.out.println("ex getResponsemsg: " + ex.getMessage() + "\n");
            }
        });
        for (SimilarityMatrix SMX : futurereturn.values()) {
            double distance = SMX.getDistance();
            /*
            System.out.println("index: " + index + "\nfutures size: " + futureslocal.values().size() + "\nScore: " + SMX.getDistance() + "\nSecondary: "
                    + SMX.getSecondaryString() + "\nPrimary: " + SMX.getPrimaryString() + "\n");
             */
            if (distance > Score) {
                Score = distance;
                SMXreturn = SMX;
            }
            index++;
        }
        System.out.println("Reached end: secondary: " + SMXreturn.getSecondaryString() + "\nPrimarY: " + SMXreturn.getPrimaryString()
                + "\nScore: " + SMXreturn.getDistance());
        return SMXreturn.getSecondaryString();
    }

    public String mostSimilar(String toBeCompared, ConcurrentMap<Integer, String> concurrentStrings) {
        int minDistance = 7;
        String similar = "";
        List<Future<ConcurrentMap<String, Integer>>> futures = new ArrayList();
        ConcurrentMap<String, Integer> futuresreturnvalues = new MapMaker().concurrencyLevel(2).makeMap();
        concurrentStrings.values().parallelStream().forEach((str) -> {
            Callable<ConcurrentMap<String, Integer>> worker = new LevenshteinDistance(toBeCompared, str);
            futures.add(executor.submit(worker));
        });
        futures.parallelStream().forEach((future) -> {
            try {
                ConcurrentMap<String, Integer> get = future.get();
                get.entrySet().forEach((str) -> {
                    futuresreturnvalues.put(str.getKey(), str.getValue());
                });
            } catch (NullPointerException | InterruptedException | ExecutionException ex) {
                System.out.println("failed future\nex: " + ex.getMessage() + "\n");
            }
        });
        for (Entry<String, Integer> entritr : futuresreturnvalues.entrySet()) {
            int distance = entritr.getValue();
            if (distance < minDistance) {
                System.out.println("distance: " + distance + "\n");
                minDistance = distance;
                similar = entritr.getKey();
            }
        }
        return similar;
    }

    public static ConcurrentMap<Integer, String> cutContent(ConcurrentMap<Integer, String> str, boolean hlStatsMsg) {
        ConcurrentMap<Integer, String> returnlist = new MapMaker().concurrencyLevel(2).makeMap();
        str.values().parallelStream().forEach(str1 -> {
            int iend = str1.indexOf("content: ");
            if (iend != -1) {
                String trs = str1.substring(iend + 9);
                returnlist.put(returnlist.size() + 1, trs.substring(0, trs.length() - 1));
            } else if (hlStatsMsg) {
                returnlist.put(returnlist.size() + 1, str1);
            }
        });
        return returnlist;
    }

    public static ConcurrentMap<Integer, String> filterContent(ConcurrentMap<Integer, String> str) {
        ConcurrentMap<Integer, String> strlistreturn = new MapMaker().concurrencyLevel(2).makeMap();
        str.values().parallelStream().forEach(str1 -> {
            if (!str1.isEmpty() && str1.length() > 3) {
                str1 = str1.trim();
                if (str1.contains("PM*")) {
                    str1 = str1.substring(str1.indexOf("PM*") + 3);
                }
                if (str1.contains("AM*")) {
                    str1 = str1.substring(str1.indexOf("AM*") + 3);
                }
                for (Character c : str1.toCharArray()) {
                    if (c == '?' || c == '°') {
                        str1 = str1.replace("?", " <:wlenny:514861023002624001> ");
                        str1 = str1.replace("°", " <:wlenny:514861023002624001> ");
                    }
                }
                if (str1.contains("(Counter-Terrorist)")) {
                    str1 = str1.replace("(Counter-Terrorist)", " ");
                }
                if (str1.contains("(Terrorist)")) {
                    str1 = str1.replace("(Terrorist)", " ");
                }
                if (str1.contains("(Spectator)")) {
                    str1 = str1.replace("(Spectator)", " ");
                }
                if (str1.contains("*DEAD*")) {
                    str1 = str1.replace("*DEAD*", " ");
                }
                if (str1.contains("{red}")) {
                    str1 = str1.replace("{red}", " ");
                }
                if (str1.contains("{orange}")) {
                    str1 = str1.replace("{orange}", " ");
                }
                if (str1.contains("{yellow}")) {
                    str1 = str1.replace("{yellow}", " ");
                }
                if (str1.contains("{green}")) {
                    str1 = str1.replace("{green}", " ");
                }
                if (str1.contains("{lightblue}")) {
                    str1 = str1.replace("{lightblue}", " ");
                }
                if (str1.contains("{blue}")) {
                    str1 = str1.replace("{blue}", " ");
                }
                if (str1.contains("{purple}")) {
                    str1 = str1.replace("{purple}", " ");
                }
                if (str1.contains("{white}")) {
                    str1 = str1.replace("{white}", " ");
                }
                if (str1.contains("{fullblue}")) {
                    str1 = str1.replace("{fullblue}", " ");
                }
                if (str1.contains("{cyan}")) {
                    str1 = str1.replace("{cyan}", " ");
                }
                if (str1.contains("{lime}")) {
                    str1 = str1.replace("{lime}", " ");
                }
                if (str1.contains("{deeppink}")) {
                    str1 = str1.replace("{deeppink}", " ");
                }
                if (str1.contains("{slategray}")) {
                    str1 = str1.replace("{slategray}", " ");
                }
                if (str1.contains("{dodgerblue}")) {
                    str1 = str1.replace("{dodgerblue}", " ");
                }
                if (str1.contains("{black}")) {
                    str1 = str1.replace("{black}", " ");
                }
                if (str1.contains("{orangered}")) {
                    str1 = str1.replace("{orangered}", " ");
                }
                if (str1.contains("{darkorchid}")) {
                    str1 = str1.replace("{darkorchid}", " ");
                }
                if (str1.contains("{pink}")) {
                    str1 = str1.replace("{pink}", " ");
                }
                if (str1.contains("{lightyellow}")) {
                    str1 = str1.replace("{lightyellow}", " ");
                }
                if (str1.contains("{chocolate}")) {
                    str1 = str1.replace("{chocolate}", " ");
                }
                if (str1.contains("{beige}")) {
                    str1 = str1.replace("{beige}", " ");
                }
                if (str1.contains("{azure}")) {
                    str1 = str1.replace("{azure}", " ");
                }
                if (str1.contains("{yellowgreen}")) {
                    str1 = str1.replace("{yellowgreen}", " ");
                }
                str1 = str1.trim();
                if (str1.length() > 2 && (!str1.startsWith("!"))) {
                    strlistreturn.put(strlistreturn.size() + 1, str1);
                }
            }
        });
        return strlistreturn;
    }

    private ConcurrentMap<Integer, String> removeSlacks(ConcurrentMap<Integer, String> str) {
        ShiftReduceParser model = getModel();
        MaxentTagger tagger = getTagger();
        ConcurrentMap<Integer, String> strreturn = new MapMaker().concurrencyLevel(2).makeMap();
        str.values().parallelStream().forEach(str1 -> {
            ConcurrentMap<Integer, String> TGWList = new MapMaker().concurrencyLevel(2).makeMap();
            DocumentPreprocessor tokenizer = null;
            try {
                tokenizer = new DocumentPreprocessor(new StringReader(str1));
            } catch (Exception ex) {
                System.out.println("failed tokenizer removeslacks: " + ex.getLocalizedMessage() + "\n");
                tokenizer = null;
            }
            if (tokenizer != null) {
                for (List<HasWord> sentence : tokenizer) {
                    int counter = 0;
                    List<TaggedWord> taggedWords;
                    List<TaggedWord> tagged1 = tagger.tagSentence(sentence);
                    Tree tree = model.apply(tagged1);
                    taggedWords = tree.taggedYield();
                    for (TaggedWord TGW : taggedWords) {
                        if (!TGWList.values().contains(TGW.tag()) && !TGW.tag().equals(":") && !TGW.word().equals(TGW.tag())) {
                            TGWList.put(TGWList.size() + 1, TGW.tag());
                            counter++;
                        }
                        if (counter > 3) {
                            int addCounter = 0;
                            ConcurrentMap<Integer, Word> wordList = new MapMaker().concurrencyLevel(2).makeMap();
                            for (Word lab : tree.yieldWords()) {
                                if (lab != null && lab.word() != null) {
                                    //System.out.println("lab: " + lab + " \n");
                                    if (!wordList.values().contains(lab) && lab.value() != null && !lab.value().equals(":")) {
                                        wordList.put(wordList.size() + 1, lab);
                                        addCounter++;
                                    }
                                }
                            }
                            if (addCounter > 3) {
                                addCounter = 0;
                                ConcurrentMap<Integer, HasWord> HWlist = new MapMaker().concurrencyLevel(2).makeMap();
                                for (HasWord HW : tree.yieldHasWord()) {
                                    if (HW != null && HW.word() != null && !HWlist.values().contains(HW)) {
                                        addCounter++;
                                        HWlist.put(HWlist.size() + 1, HW);
                                    }
                                }
                                if (addCounter > 3) {
                                    boolean tooclosematch = false;
                                    Collection<String> values = stringCache.values();
                                    for (String strVals : values) {
                                        LevenshteinDistance leven = new LevenshteinDistance(strVals, str1);
                                        double Distance = leven.computeLevenshteinDistance();
                                        int maxpermittedDistance = 2;
                                        if (Distance < maxpermittedDistance) {
                                            tooclosematch = true;
                                            break;
                                        }
                                    }
                                    if (!tooclosematch) {
                                        strreturn.put(strreturn.size() + 1, str1);
                                        System.out.println("adding strreturn str1: " + str1 + "\n");
                                    }
                                }
                            }
                            break;
                        }
                    }
                    if (counter > 3) {
                        break;
                    }
                }
            }
        });
        return strreturn;
    }

    private ConcurrentMap<Integer, String> verifyCalculationFitness(ConcurrentMap<Integer, String> strmap) {
        ConcurrentMap<Integer, String> returnmap = new MapMaker().concurrencyLevel(2).makeMap();
        ConcurrentMap<String, Annotation> pipelineAnnotateCachelcl = new MapMaker().concurrencyLevel(2).makeMap();
        ConcurrentMap<String, Annotation> pipelineSentimentAnnotateCachelcl = new MapMaker().concurrencyLevel(2).makeMap();
        ConcurrentMap<String, Annotation> jmweAnnotateCachelcl = new MapMaker().concurrencyLevel(2).makeMap();
        ConcurrentMap<String, Annotation> jmweAnnotation = PipelineJMWESingleton.INSTANCE.getJMWEAnnotation(strmap.values());
        for (Entry<String, Annotation> jmweitr : jmweAnnotation.entrySet()) {
            jmweAnnotateCachelcl.put(jmweitr.getKey(), jmweitr.getValue());
        }
        strmap.values().parallelStream().forEach(strCache -> {
            Annotation strAnno = new Annotation(strCache);
            pipeline.annotate(strAnno);
            pipelineAnnotateCachelcl.put(strCache, strAnno);
            Annotation strAnno2 = new Annotation(strCache);
            pipelineSentiment.annotate(strAnno2);
            pipelineSentimentAnnotateCachelcl.put(strCache, strAnno2);
            System.out.println("normal annotating strCache: " + strCache + "\n");
        });
        final ConcurrentMap<Integer, String> allStrings;
        if (!stringCache.isEmpty()) {
            allStrings = stringCache;
        } else {
            allStrings = strmap;
        }
        ConcurrentMap<Integer, Future<SimilarityMatrix>> futures = new MapMaker().concurrencyLevel(2).makeMap();
        strmap.values().parallelStream().forEach((str) -> {
            for (String str1 : allStrings.values()) {
                Callable<SimilarityMatrix> worker = new SentimentAnalyzerTest(str, str1, new SimilarityMatrix(str, str1),
                        jmweAnnotateCachelcl.get(str), jmweAnnotateCachelcl.get(str1), pipelineAnnotateCachelcl.get(str),
                        pipelineAnnotateCachelcl.get(str1), pipelineSentimentAnnotateCachelcl.get(str),
                        pipelineSentimentAnnotateCachelcl.get(str1));
                futures.put(futures.size() + 1, executor.submit(worker));
                System.out.println("futures size in verify calcs: " + futures.size() + "\n");
            }
        });
        futures.values().parallelStream().forEach((future) -> {
            SimilarityMatrix get;
            //turning from 20 to 5 might be risky?
            try {
                get = future.get(5, TimeUnit.SECONDS);
                String addStr = get.getPrimaryString();
                returnmap.put(returnmap.size() + 1, addStr);
                System.out.println("returnmap adding: " + addStr + "\n");
            } catch (InterruptedException | ExecutionException | TimeoutException ex) {
                System.out.println("failed verification: " + ex.getMessage() + "\n");
            }
        });
        jmweAnnotation = PipelineJMWESingleton.INSTANCE.getJMWEAnnotation(returnmap.values());
        for (Entry<String, Annotation> jmweitr : jmweAnnotation.entrySet()) {
            jmweAnnotationCache.put(jmweitr.getKey(), jmweitr.getValue());
        }
        returnmap.values().parallelStream().forEach(strCache -> {
            stringCache.put(stringCache.size() + 1, strCache);
            System.out.println("str annotation pipeline pipelinesentiment: " + strCache + "\n");
            Annotation strAnno = new Annotation(strCache);
            pipeline.annotate(strAnno);
            pipelineAnnotationCache.put(strCache, strAnno);
            Annotation strAnno2 = new Annotation(strCache);
            pipelineSentiment.annotate(strAnno2);
            pipelineSentimentAnnotationCache.put(strCache, strAnno2);
        });
        return returnmap;
    }
}
