/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package FunctionLayer;

import DataLayer.DataMapper;
import FunctionLayer.StanfordParser.SentimentAnalyzerTest;
import com.google.common.base.Stopwatch;
import com.google.common.collect.MapMaker;
import edu.stanford.nlp.ie.AbstractSequenceClassifier;
import edu.stanford.nlp.ie.crf.CRFClassifier;
import edu.stanford.nlp.ling.CoreLabel;
import edu.stanford.nlp.ling.HasWord;
import edu.stanford.nlp.ling.TaggedWord;
import edu.stanford.nlp.ling.Word;
import edu.stanford.nlp.parser.lexparser.LexicalizedParser;
import edu.stanford.nlp.parser.shiftreduce.ShiftReduceParser;
import edu.stanford.nlp.pipeline.Annotation;
import edu.stanford.nlp.pipeline.StanfordCoreNLP;
import edu.stanford.nlp.process.DocumentPreprocessor;
import edu.stanford.nlp.tagger.maxent.MaxentTagger;
import edu.stanford.nlp.trees.GrammaticalStructureFactory;
import edu.stanford.nlp.trees.Tree;
import edu.stanford.nlp.trees.TreebankLanguagePack;
import java.io.IOException;
import java.io.StringReader;
import java.sql.SQLException;
import java.util.Collection;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Properties;
import java.util.Random;
import java.util.concurrent.Callable;
import java.util.concurrent.ConcurrentMap;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.ForkJoinPool;
import java.util.concurrent.Future;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.TimeoutException;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 *
 * @author install1
 */
public class MYSQLDatahandler {

    public static final long EXPIRE_TIME_IN_SECONDS = TimeUnit.SECONDS.convert(6, TimeUnit.MINUTES);
    public static final long EXPIRE_TIME_IN_SECONDS1 = TimeUnit.SECONDS.convert(10, TimeUnit.HOURS);
    public static MYSQLDatahandler instance = new MYSQLDatahandler();
    private volatile boolean refreshMatrixFromDB;
    private static int secondaryIterator = 0;
    private final ConcurrentMap<Integer, String> stringCache;
    private static ConcurrentMap<String, Annotation> pipelineAnnotationCache;
    private static ConcurrentMap<String, Annotation> pipelineSentimentAnnotationCache;
    private static ConcurrentMap<String, Annotation> jmweAnnotationCache;
    private LinkedHashMap<String, LinkedHashMap<String, Double>> lHMSMX = new LinkedHashMap();
    private final Stopwatch stopwatch;
    private final Stopwatch stopwatch1;
    private ForkJoinPool executor;
    private static String shiftReduceParserPath = "edu/stanford/nlp/models/srparser/englishSR.ser.gz";
    private static String sentimentModel = "edu/stanford/nlp/models/sentiment/sentiment.ser.gz";
    private static String lexParserEnglishRNN = "edu/stanford/nlp/models/lexparser/englishRNN.ser.gz";
    private static String taggerPath = "edu/stanford/nlp/models/pos-tagger/english-left3words/english-left3words-distsim.tagger";
    private static String nerModel = "edu/stanford/nlp/models/ner/english.all.3class.distsim.crf.ser.gz";
    private static String jmweIndexData = "/home/javatests/lib/mweindex_wordnet3.0_semcor1.6.data"; // ./lib/mweindex_wordnet3.0_semcor1.6.data
    private static String jmweIndexDataLocalTest = "E:/java8/Projects/mweindex_wordnet3.0_semcor1.6.data";
    private static MaxentTagger tagger;
    private static ShiftReduceParser model;
    private static String[] options = {"-maxLength", "100"};
    private static Properties props = new Properties();
    private static Properties propsSentiment = new Properties();
    private static Properties propsJMWE = new Properties();
    private static GrammaticalStructureFactory gsf;
    private static LexicalizedParser lp;
    private static TreebankLanguagePack tlp;
    private static AbstractSequenceClassifier<CoreLabel> classifier;
    // set up Stanford CoreNLP pipeline
    private static StanfordCoreNLP pipeline;
    private static StanfordCoreNLP pipelineSentiment;
    private static StanfordCoreNLP pipelineJMWE;

    public MYSQLDatahandler() {
        this.stopwatch = Stopwatch.createUnstarted();
        this.stopwatch1 = Stopwatch.createStarted();
        this.stringCache = new MapMaker().concurrencyLevel(2).makeMap();
        //cant sadly just have one annotation for every pipelines, one pipeline per annotation is required
        this.jmweAnnotationCache = new MapMaker().concurrencyLevel(2).makeMap();
        this.pipelineAnnotationCache = new MapMaker().concurrencyLevel(2).makeMap();
        this.pipelineSentimentAnnotationCache = new MapMaker().concurrencyLevel(2).makeMap();
    }

    public void shiftReduceParserInitiate() {
        try {
            classifier = CRFClassifier.getClassifierNoExceptions(nerModel);
        } catch (ClassCastException ex) {
            Logger.getLogger(MYSQLDatahandler.class.getName()).log(Level.SEVERE, null, ex);
        }
        model = ShiftReduceParser.loadModel(shiftReduceParserPath, options);
        tagger = new MaxentTagger(taggerPath);
        lp = LexicalizedParser.loadModel(lexParserEnglishRNN, options);
        tlp = lp.getOp().langpack();
        gsf = tlp.grammaticalStructureFactory();
        // set up pipeline properties
        props.setProperty("parse.model", shiftReduceParserPath);
        props.setProperty("parse.maxlen", "100");
        props.setProperty("parse.binaryTrees", "true");
        props.setProperty("annotators", "tokenize,ssplit,pos,lemma,parse");
        propsSentiment.setProperty("parse.model", lexParserEnglishRNN);
        propsSentiment.setProperty("ner.model", nerModel);
        propsSentiment.setProperty("sentiment.model", sentimentModel);
        propsSentiment.setProperty("parse.maxlen", "100");
        propsSentiment.setProperty("annotators", "tokenize,ssplit,pos,parse,depparse,sentiment"); //coref too expensive memorywise
        propsJMWE.setProperty("customAnnotatorClass.jmwe", "edu.stanford.nlp.pipeline.JMWEAnnotator");
        propsJMWE.setProperty("customAnnotatorClass.jmwe.verbose", "false");
        propsJMWE.setProperty("customAnnotatorClass.jmwe.underscoreReplacement", "-");
        propsJMWE.setProperty("customAnnotatorClass.jmwe.indexData", jmweIndexData); //jmweIndexDataLocalTest jmweIndexData
        propsJMWE.setProperty("customAnnotatorClass.jmwe.detector", "Exhaustive");
        //"Consecutive", "Exhaustive", "ProperNouns", "Complex" and "CompositeConsecutiveProperNouns"
        propsJMWE.setProperty("annotators", "tokenize, ssplit, pos, lemma, jmwe");
        // set up pipeline
        pipeline = new StanfordCoreNLP(props);
        pipelineSentiment = new StanfordCoreNLP(propsSentiment);
        pipelineJMWE = new StanfordCoreNLP(propsJMWE);
    }

    public static AbstractSequenceClassifier<CoreLabel> getClassifier() {
        return classifier;
    }

    public static void setClassifier(AbstractSequenceClassifier<CoreLabel> classifier) {
        MYSQLDatahandler.classifier = classifier;
    }

    public void updateStringCache() {
        try {
            checkIfUpdateStrings();
        } catch (CustomError ex) {
            Logger.getLogger(MYSQLDatahandler.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    public void instantiateExecutor() {
        this.executor = new ForkJoinPool(Runtime.getRuntime().availableProcessors(),
                ForkJoinPool.defaultForkJoinWorkerThreadFactory,
                null, false); //true
    }

    public static GrammaticalStructureFactory getGsf() {
        return gsf;
    }

    public static StanfordCoreNLP getPipeline() {
        return pipeline;
    }

    public static StanfordCoreNLP getPipelineSentiment() {
        return pipelineSentiment;
    }

    public static MaxentTagger getTagger() {
        return tagger;
    }

    public static ShiftReduceParser getModel() {
        return model;
    }

    private Map<Integer, String> getCache() throws SQLException, IOException, CustomError {
        return DataMapper.getAllStrings();
    }

    public int getlHMSMXSize() {
        return lHMSMX.size();
    }

    public int getstringCacheSize() {
        return stringCache.size();
    }

    public void initiateMYSQL() throws SQLException, IOException {
        try {
            DataMapper.createTables();
            stringCache.putAll(getCache());
            addHLstatsMessages();
            lHMSMX = DataMapper.getAllRelationScores();
        } catch (CustomError ex) {
            Logger.getLogger(MYSQLDatahandler.class
                    .getName()).log(Level.SEVERE, null, ex);
        }
    }

    private void addHLstatsMessages() {
        ConcurrentMap<Integer, String> hlStatsMessages = DataMapper.getHLstatsMessages();
        ConcurrentMap<Integer, String> strCacheLocal = stringCache;
        int hardcap = 300;
        int counter = 0;
        for (String str : hlStatsMessages.values()) {
            if (!str.startsWith("!")) {
                boolean present = false;
                for (String str1 : strCacheLocal.values()) {
                    if (str.equals(str1)) {
                        present = true;
                        break;
                    }
                }
                if (!present) {
                    MessageResponseHandler.getMessage(str);
                }
            }
            if (counter >= hardcap) {
                break;
            }
            counter++;
        }
    }

    public void instantiateAnnotationMap() {
        for (String str : stringCache.values()) {
            System.out.println("str annotation jmwe: " + str + "\n");
            Annotation strAnno = new Annotation(str);
            pipelineJMWE.annotate(strAnno);
            jmweAnnotationCache.put(str, strAnno);
        }
        for (String str : stringCache.values()) {
            System.out.println("str annotation pipeline: " + str + "\n");
            Annotation strAnno = new Annotation(str);
            pipeline.annotate(strAnno);
            pipelineAnnotationCache.put(str, strAnno);
        }
        for (String str : stringCache.values()) {
            System.out.println("str annotation pipelinesentiment: " + str + "\n");
            Annotation strAnno = new Annotation(str);
            pipelineSentiment.annotate(strAnno);
            pipelineSentimentAnnotationCache.put(str, strAnno);
        }
    }

    public synchronized void checkIfUpdateMatrixes() {
        refreshMatrixFromDB = false;
        int counter = 0;
        if (stopwatch1.elapsed(TimeUnit.SECONDS) >= EXPIRE_TIME_IN_SECONDS1) {
            refreshMatrixFromDB = true;
            lHMSMX = DataMapper.getAllRelationScores();
            stopwatch1.reset();
        }
        //requiring atleast 10 entries ensures no issues in case of empty stringcache
        if (stringCache.values().size() > 10 && !refreshMatrixFromDB) {
            ConcurrentMap<Integer, String> stringCachelocal = stringCache;
            int selectUpdate = -1;
            LinkedHashMap<String, LinkedHashMap<String, Double>> LHMSMXLocal = lHMSMX;
            int ij2 = 0;
            for (String str : stringCachelocal.values()) {
                boolean updatepresent = false;
                for (String strlocal : LHMSMXLocal.keySet()) {
                    if (strlocal.equals(str)) {
                        updatepresent = true;
                        break;
                    }
                }
                if (!updatepresent) {
                    selectUpdate = ij2;
                    break;
                }
                ij2++;
            }
            if (selectUpdate == -1 || selectUpdate + 1 == stringCachelocal.size()) {
                int valueSize = stringCachelocal.size();
                if (secondaryIterator + 1 >= valueSize) {
                    secondaryIterator = 0;
                }
                selectUpdate = secondaryIterator;
                secondaryIterator++;
            }
            ConcurrentMap<Integer, String> strIndexNavigator = new MapMaker().concurrencyLevel(2).makeMap();
            String get = stringCachelocal.getOrDefault(selectUpdate, null);
            if (get == null) {
                get = stringCachelocal.get(new Random().nextInt(stringCachelocal.size() - 1));
            }
            strIndexNavigator.put(0, get);
            ConcurrentMap<Integer, SimilarityMatrix> matrixUpdateList = new MapMaker().concurrencyLevel(2).makeMap();
            ConcurrentMap<Integer, Future<SimilarityMatrix>> futures = new MapMaker().concurrencyLevel(2).makeMap();
            strIndexNavigator.values().forEach((str) -> {
                stringCachelocal.values().stream().filter((str1) -> (!str.equals(str1))).forEachOrdered((str1) -> {
                    boolean present = false;
                    LinkedHashMap<String, Double> orDefault = lHMSMX.getOrDefault(str, null);
                    if (orDefault != null) {
                        for (String strkey : orDefault.keySet()) {
                            if (strkey.equals(str1)) {
                                present = true;
                                break;
                            }
                        }
                    }
                    if (!present) {
                        orDefault = lHMSMX.getOrDefault(str1, null);
                        if (orDefault != null) {
                            for (String strkey : orDefault.keySet()) {
                                if (strkey.equals(str)) {
                                    present = true;
                                    break;
                                }
                            }
                        }
                    }
                    if (!present) {
                        LinkedHashMap<String, Double> orDefault1 = lHMSMX.getOrDefault(str, null);
                        if (orDefault1 == null) {
                            orDefault1 = new LinkedHashMap<String, Double>();
                        }
                        orDefault1.put(str1, 0.0);
                        lHMSMX.put(str, orDefault1);
                        SimilarityMatrix SMX = new SimilarityMatrix(str, str1);
                        Callable<SimilarityMatrix> worker = new SentimentAnalyzerTest(str, str1, SMX, jmweAnnotationCache.get(str),
                                jmweAnnotationCache.get(str1), pipelineAnnotationCache.get(str), pipelineAnnotationCache.get(str1),
                                pipelineSentimentAnnotationCache.get(str), pipelineSentimentAnnotationCache.get(str1));
                        futures.put(futures.size() + 1, executor.submit(worker));
                    }
                });
            });
            System.out.println("finished worker assignment, futures size: " + futures.size() + "\n");
            for (Future<SimilarityMatrix> future : futures.values()) {
                SimilarityMatrix SMX = new SimilarityMatrix("", "");
                try {
                    SMX = future.get(20, TimeUnit.SECONDS);
                } catch (InterruptedException | ExecutionException | TimeoutException ex) {
                    Logger.getLogger(MYSQLDatahandler.class.getName()).log(Level.SEVERE, null, ex);
                    SMX = null;
                }
                System.out.println("counter: " + counter + "\nSMX: " + SMX == null ? "SMX null\n" : "SMX not null\n");
                counter++;
                if (SMX != null) {
                    LinkedHashMap<String, Double> getFuture = lHMSMX.getOrDefault(SMX.getPrimaryString(), null);
                    getFuture.put(SMX.getSecondaryString(), SMX.getDistance());
                    lHMSMX.put(SMX.getPrimaryString(), getFuture);
                    matrixUpdateList.put(matrixUpdateList.size() + 1, SMX);
                }
            }
            try {
                if (!matrixUpdateList.isEmpty()) {
                    DataMapper.insertSementicMatrixes(matrixUpdateList);
                    System.out.println("finished datamapper semetic insert");
                }
            } catch (CustomError ex) {
                Logger.getLogger(MYSQLDatahandler.class
                        .getName()).log(Level.SEVERE, null, ex);
            }
        }
    }

    public synchronized void checkIfUpdateStrings() throws CustomError {
        if (stopwatch.elapsed(TimeUnit.SECONDS) >= EXPIRE_TIME_IN_SECONDS || !stopwatch.isRunning()) {
            ConcurrentMap<Integer, String> str = MessageResponseHandler.getStr();
            str = cutContent(str);
            str = filterContent(str);
            str = removeSlacks(str);
            str = verifyCalculationFitness(str);
            try {
                DataMapper.InsertMYSQLStrings(str);
            } catch (CustomError ex) {
                Logger.getLogger(MYSQLDatahandler.class
                        .getName()).log(Level.SEVERE, null, ex);
            }
            MessageResponseHandler.setStr(new MapMaker().concurrencyLevel(2).makeMap());
            int j = stringCache.size() + 1;
            for (String str1 : str.values()) {
                stringCache.put(j, str1);
                j++;
                Annotation strAnno = new Annotation(str1);
                pipelineJMWE.annotate(strAnno);
                jmweAnnotationCache.put(str1, strAnno);
                strAnno = new Annotation(str1);
                pipeline.annotate(strAnno);
                pipelineAnnotationCache.put(str1, strAnno);
                strAnno = new Annotation(str1);
                pipelineSentiment.annotate(strAnno);
                pipelineSentimentAnnotationCache.put(str1, strAnno);
            }
            if (!stopwatch.isRunning()) {
                stopwatch.start();
            } else {
                stopwatch.reset();
            }
        }
    }

    public synchronized String getResponseMsg(String str) throws CustomError {
        str = str.trim();
        if (str.startsWith("<@")) {
            str = str.substring(str.indexOf("> ") + 2);
        }
        final LinkedHashMap<String, LinkedHashMap<String, Double>> LHMSMXLocal = lHMSMX;
        ConcurrentMap<Integer, String> strArrs = stringCache;
        double Score = -10000;
        SimilarityMatrix SMXreturn = new SimilarityMatrix("", "");
        System.out.println("pre mostSimilarSTR \n");
        String mostSimilarSTR = mostSimilar(str, strArrs);
        if (mostSimilarSTR != null) {
            System.out.println("mostSimilarSTR; " + mostSimilarSTR + "\n");
            LinkedHashMap<String, Double> orDefault = LHMSMXLocal.getOrDefault(mostSimilarSTR, null);
            if (orDefault != null) {
                for (Entry<String, Double> entrySet : orDefault.entrySet()) {
                    double smxDistance = entrySet.getValue();
                    if (smxDistance > Score) {
                        Score = smxDistance;
                        SMXreturn = new SimilarityMatrix(mostSimilarSTR, entrySet.getKey(), smxDistance);
                    }
                }
            }
            for (Entry<String, LinkedHashMap<String, Double>> values1 : LHMSMXLocal.entrySet()) {
                LinkedHashMap<String, Double> value = values1.getValue();
                for (Entry<String, Double> keystr : value.entrySet()) {
                    if (keystr.getKey().equals(mostSimilarSTR)) {
                        double smxDistance = keystr.getValue();
                        if (smxDistance > Score) {
                            Score = smxDistance;
                            SMXreturn = new SimilarityMatrix(values1.getKey(), keystr.getKey(), smxDistance);
                        }
                    }
                }
            }
            if (!SMXreturn.getPrimaryString().isEmpty()) {
                if (SMXreturn.getPrimaryString().equals(mostSimilarSTR)) {
                    return SMXreturn.getSecondaryString();
                } else {
                    return SMXreturn.getPrimaryString();
                }
            }
        }
        System.out.println("none within 8 range");
        ConcurrentMap<Integer, String> strCache = stringCache;
        ConcurrentMap<Integer, Future<SimilarityMatrix>> futureslocal = new MapMaker().concurrencyLevel(2).makeMap();
        for (String str1 : strCache.values()) {
            if (!str.equals(str1)) {
                SimilarityMatrix SMX = new SimilarityMatrix(str, str1);
                Callable<SimilarityMatrix> worker = new SentimentAnalyzerTest(str, str1, SMX,
                        jmweAnnotationCache.get(str), jmweAnnotationCache.get(str1), pipelineAnnotationCache.get(str),
                        pipelineAnnotationCache.get(str1), pipelineSentimentAnnotationCache.get(str),
                        pipelineSentimentAnnotationCache.get(str1));
                futureslocal.put(futureslocal.size() + 1, executor.submit(worker));
            }
        }
        int index = 0;
        for (Future<SimilarityMatrix> future : futureslocal.values()) {
            SimilarityMatrix SMX = new SimilarityMatrix("", "");
            try {
                SMX = future.get(20, TimeUnit.SECONDS);
            } catch (InterruptedException | ExecutionException | TimeoutException ex) {
                System.out.println("ex getResponsemsg: " + ex.getMessage() + "\n");
                SMX = null;
            }
            if (SMX != null) {
                double distance = SMX.getDistance();
                System.out.println("index: " + index + "\nfutures size: " + futureslocal.values().size() + "\nScore: " + SMX.getDistance() + "\nSecondary: "
                        + SMX.getSecondaryString() + "\nPrimary: " + SMX.getPrimaryString() + "\n");
                if (distance > Score) {
                    Score = distance;
                    SMXreturn = SMX;
                }
            }
            index++;
        }
        System.out.println("Reached end: secondary: " + SMXreturn.getSecondaryString() + "\nPrimarY: " + SMXreturn.getPrimaryString()
                + "\nScore: " + SMXreturn.getDistance());
        return SMXreturn.getSecondaryString();
    }

    public String mostSimilar(String toBeCompared, ConcurrentMap<Integer, String> concurrentStrings) {
        int minDistance = 8;
        String similar = "";
        ConcurrentMap<Integer, Future<Entry<String, Integer>>> futures = new MapMaker().concurrencyLevel(2).makeMap();
        for (String str : concurrentStrings.values()) {
            Callable<Entry<String, Integer>> worker = new LevenshteinDistance(toBeCompared, str);
            futures.put(futures.size() + 1, executor.submit(worker));
        }
        for (Future<Entry<String, Integer>> future : futures.values()) {
            try {
                Entry<String, Integer> futureEntry = future.get();
                int distance = futureEntry.getValue();
                if (distance < minDistance) {
                    System.out.println("distance: " + distance + "\n");
                    minDistance = distance;
                    similar = futureEntry.getKey();
                }
            } catch (NullPointerException | InterruptedException | ExecutionException ex) {
                System.out.println("failed future\n");
            }
        }
        return similar;
    }

    public static ConcurrentMap<Integer, String> cutContent(ConcurrentMap<Integer, String> str) {
        ConcurrentMap<Integer, String> returnlist = new MapMaker().concurrencyLevel(2).makeMap();
        for (String str1 : str.values()) {
            int iend = str1.indexOf("content: ");
            if (iend != -1) {
                String trs = str1.substring(iend + 9);
                returnlist.put(returnlist.size() + 1, trs.substring(0, trs.length() - 1));
            }
        }
        return returnlist;
    }

    public static ConcurrentMap<Integer, String> filterContent(ConcurrentMap<Integer, String> str) {
        ConcurrentMap<Integer, String> strlistreturn = new MapMaker().concurrencyLevel(2).makeMap();
        for (String str1 : str.values()) {
            if (str1.isEmpty() || str1.length() < 3) {
                continue;
            }
            str1 = str1.trim();
            if (str1.contains("PM*")) {
                str1 = str1.substring(str1.indexOf("PM*") + 5);
            }
            if (str1.contains("AM*")) {
                str1 = str1.substring(str1.indexOf("AM*") + 5);
            }
            for (Character c : str1.toCharArray()) {
                if (c == '?' || c == '°') {
                    str1 = str1.replace("?", " <:wlenny:514861023002624001> ");
                    str1 = str1.replace("°", " <:wlenny:514861023002624001> ");
                }
            }
            if (str1.contains("(Counter-Terrorist)")) {
                str1 = str1.replace("(Counter-Terrorist)", " ");
            }
            if (str1.contains("(Terrorist)")) {
                str1 = str1.replace("(Terrorist)", " ");
            }
            if (str1.contains("(Spectator)")) {
                str1 = str1.replace("(Spectator)", " ");
            }
            if (str1.contains("*DEAD*")) {
                str1 = str1.replace("*DEAD*", " ");
            }
            if (str1.contains("{red}")) {
                str1 = str1.replace("{red}", " ");
            }
            if (str1.contains("{orange}")) {
                str1 = str1.replace("{orange}", " ");
            }
            if (str1.contains("{yellow}")) {
                str1 = str1.replace("{yellow}", " ");
            }
            if (str1.contains("{green}")) {
                str1 = str1.replace("{green}", " ");
            }
            if (str1.contains("{lightblue}")) {
                str1 = str1.replace("{lightblue}", " ");
            }
            if (str1.contains("{blue}")) {
                str1 = str1.replace("{blue}", " ");
            }
            if (str1.contains("{purple}")) {
                str1 = str1.replace("{purple}", " ");
            }
            if (str1.contains("{white}")) {
                str1 = str1.replace("{white}", " ");
            }
            if (str1.contains("{fullblue}")) {
                str1 = str1.replace("{fullblue}", " ");
            }
            if (str1.contains("{cyan}")) {
                str1 = str1.replace("{cyan}", " ");
            }
            if (str1.contains("{lime}")) {
                str1 = str1.replace("{lime}", " ");
            }
            if (str1.contains("{deeppink}")) {
                str1 = str1.replace("{deeppink}", " ");
            }
            if (str1.contains("{slategray}")) {
                str1 = str1.replace("{slategray}", " ");
            }
            if (str1.contains("{dodgerblue}")) {
                str1 = str1.replace("{dodgerblue}", " ");
            }
            if (str1.contains("{black}")) {
                str1 = str1.replace("{black}", " ");
            }
            if (str1.contains("{orangered}")) {
                str1 = str1.replace("{orangered}", " ");
            }
            if (str1.contains("{darkorchid}")) {
                str1 = str1.replace("{darkorchid}", " ");
            }
            if (str1.contains("{pink}")) {
                str1 = str1.replace("{pink}", " ");
            }
            if (str1.contains("{lightyellow}")) {
                str1 = str1.replace("{lightyellow}", " ");
            }
            if (str1.contains("{chocolate}")) {
                str1 = str1.replace("{chocolate}", " ");
            }
            if (str1.contains("{beige}")) {
                str1 = str1.replace("{beige}", " ");
            }
            if (str1.contains("{azure}")) {
                str1 = str1.replace("{azure}", " ");
            }
            if (str1.contains("{yellowgreen}")) {
                str1 = str1.replace("{yellowgreen}", " ");
            }
            str1 = str1.trim();
            if (str1.length() > 2 && (!str1.startsWith("!"))) {
                strlistreturn.put(strlistreturn.size() + 1, str1);
            }
        }
        return strlistreturn;
    }

    private ConcurrentMap<Integer, String> removeSlacks(ConcurrentMap<Integer, String> str) {
        ShiftReduceParser model = getModel();
        MaxentTagger tagger = getTagger();
        List<TaggedWord> taggedWords;
        ConcurrentMap<Integer, String> strreturn = new MapMaker().concurrencyLevel(2).makeMap();
        for (String str1 : str.values()) {
            int counter = 0;
            ConcurrentMap<Integer, String> TGWList = new MapMaker().concurrencyLevel(2).makeMap();
            DocumentPreprocessor tokenizer = new DocumentPreprocessor(new StringReader(str1));
            for (List<HasWord> sentence : tokenizer) {
                List<TaggedWord> tagged1 = tagger.tagSentence(sentence);
                Tree tree = model.apply(tagged1);
                taggedWords = tree.taggedYield();
                for (TaggedWord TGW : taggedWords) {
                    if (!TGWList.values().contains(TGW.tag()) && !TGW.tag().equals(":") && !TGW.word().equals(TGW.tag())) {
                        TGWList.put(TGWList.size() + 1, TGW.tag());
                        counter++;
                    }
                    if (counter > 3) {
                        int addCounter = 0;
                        ConcurrentMap<Integer, Word> wordList = new MapMaker().concurrencyLevel(2).makeMap();
                        for (Word lab : tree.yieldWords()) {
                            if (lab != null && lab.word() != null) {
                                //System.out.println("lab: " + lab + " \n");
                                if (!wordList.values().contains(lab) && lab.value() != null && !lab.value().equals(":")) {
                                    wordList.put(wordList.size() + 1, lab);
                                    addCounter++;
                                }
                            }
                        }
                        if (addCounter > 3) {
                            addCounter = 0;
                            ConcurrentMap<Integer, HasWord> HWlist = new MapMaker().concurrencyLevel(2).makeMap();
                            for (HasWord HW : tree.yieldHasWord()) {
                                if (HW != null && HW.word() != null && !HWlist.values().contains(HW)) {
                                    addCounter++;
                                    HWlist.put(HWlist.size() + 1, HW);
                                }
                            }
                            if (addCounter > 3) {
                                boolean tooclosematch = false;
                                Collection<String> values = stringCache.values();
                                for (String strVals : values) {
                                    LevenshteinDistance leven = new LevenshteinDistance(strVals, str1);
                                    double Distance = leven.computeLevenshteinDistance();
                                    int maxpermittedDistance = 2;
                                    if (Distance < maxpermittedDistance) {
                                        tooclosematch = true;
                                        break;
                                    }
                                }
                                if (!tooclosematch) {
                                    strreturn.put(strreturn.size() + 1, str1);
                                }
                            }
                        }
                        break;
                    }
                }
                if (counter > 3) {
                    break;
                }
            }
        }
        return strreturn;
    }

    private ConcurrentMap<Integer, String> verifyCalculationFitness(ConcurrentMap<Integer, String> strmap) {
        ConcurrentMap<Integer, String> returnmap = new MapMaker().concurrencyLevel(2).makeMap();
        ConcurrentMap<Integer, String> allStrings = stringCache;
        int intervalAprove = allStrings.values().size() / 10;
        for (String str : strmap.values()) {
            int counter = 0;
            boolean calculationIssues = false;
            ConcurrentMap<Integer, Future<SimilarityMatrix>> futures = new MapMaker().concurrencyLevel(2).makeMap();
            int ij2 = 0;
            if (allStrings.isEmpty()) {
                allStrings = strmap;
            }
            for (String strValues : allStrings.values()) {
                if (ij2 > intervalAprove) {
                    break;
                }
                Callable<SimilarityMatrix> worker = new SentimentAnalyzerTest(str, strValues, new SimilarityMatrix(str, strValues),
                        jmweAnnotationCache.get(str), jmweAnnotationCache.get(strValues), pipelineAnnotationCache.get(str),
                        pipelineAnnotationCache.get(strValues), pipelineSentimentAnnotationCache.get(str),
                        pipelineSentimentAnnotationCache.get(strValues));
                futures.put(futures.size() + 1, executor.submit(worker));
                ij2++;
            }
            for (Future<SimilarityMatrix> future : futures.values()) {
                try {
                    SimilarityMatrix get = future.get(20, TimeUnit.SECONDS);
                    if (get == null) {
                        System.out.println("counter timeouts: " + counter + "\n");
                        counter++;
                        if (counter >= 10) {
                            calculationIssues = true;
                            break;
                        }
                    }
                } catch (InterruptedException | ExecutionException | TimeoutException ex) {
                    System.out.println("counter timeouts: " + counter + "\n");
                    counter++;
                    if (counter >= 10) {
                        calculationIssues = true;
                        break;
                    }
                }
            }
            if (!calculationIssues) {
                returnmap.put(returnmap.size() + 1, str);
            }
        }
        return returnmap;
    }
}
